# RCS: @(#) $Id: mdtm.tcl,v 1.1 2003/11/13 15:59:56 barras Exp $

# rttm.tcl - extension of the Transcriber program
# Copyright (C) 2003, LIMSI
# distributed under the GNU General Public License (see COPYING file)

namespace eval ::convert::rttm {

  variable msg "NIST .rttm data"
  variable ext ".rttm"

  # only needed for compatibility with version <1.4.6
  proc readSegmt {content} {return [lindex [lindex [readSegmtSet $content] 0] 0]}
  if {[info commands ::ColorMap] == ""} {proc ::ColorMap c {return}}

  proc readSegmtSet {content args} {
    global v
    if {[info exists v(sig,name)]} {
      set sid [file tail [file root $v(sig,name)]]
    } else {
      set sid ""
    }
    array set spk {}
    array set gnd {}
    foreach line [split $content "\n"] {
      if {$line == "" || [string match ";;*" $line]} continue
      if {[scan $line "%s%s%s%s%s%s%s%s%s" type id ch begin len wrd subtype speaker conf] >= 8} {
	# filter on signal id if available, else choose first id met
	if {$sid == ""} {
	  set sid $id
	} elseif {$id != $sid} {
	  continue
	}
	set type [string toupper $type]
	if {[string is double $begin] && [string is double $len]} {
	  set end [expr $begin+$len]
	} elseif {$type == "SPEAKER" || $type == "LEXEME" || $type == "NON-LEX"} {
	  puts stderr "Warning - wrong time codes in line '$line' during .rttm parsing"
	  continue
	}
	# currently only process SPKR-INFO, SPEAKER and LEXEME/NON-LEX tags
	switch -- [string toupper $type] {
	  "SPKR-INFO" {
	    set gender($speaker) [string tolower $subtype]
	  }
	  "SPEAKER" {
	    set col [ColorMap $speaker]
	    lappend spk($ch) [list $begin $end $speaker $col]

	    set val "?"
	    set col "#00cc00"
	    if {[info exists gender($speaker)]} {
	      switch -- $gender($speaker) {
		"adult_male" { set val "Male"; set col "#00aaff" }
		"adult_female" { set val "Female"; set col "#f67000"}
		"child" { set val "Child"; set col green}
	      }
	    }
	    lappend gnd($ch) [list $begin $end $val $col]
	  }
	  "LEXEME" {
	    set col {}
	    lappend lex($ch) [list $begin $end $wrd $col]
	  }
	  "NON-LEX" {
	    set col grey
	    lappend lex($ch) [list $begin $end $wrd $col]
	  }
	}

      } else {
	puts stderr "Warning - line '$line' ignored from .rttm parsing"
      }
    }
    set result {}
    foreach ch [lsort [array names spk]] {
      lappend result [list $spk($ch) "RTTM speaker (chn $ch)"]
      lappend result [list $gnd($ch) "RTTM gender (chn $ch)" 0]
      if {[info exists lex($ch)]} {
	lappend result [list $lex($ch) "RTTM lex/non-lex (chn $ch)" 0]
      }
    }
    if {[llength $result] == 0} {
      puts stderr "Warning - no line matched $sid basename during .rttm parsing"
    }
    return $result
  }
}